@@ -59,6 +59,8 @@ export class WorldModel {
   ui: {
     wrapper: HTMLDivElement;
     stage?: Konva.Stage;
+    stageHeight: number;
+    stageWidth: number;
     layers: {
       main: Konva.Layer;
       line: Konva.Layer;
@@ -99,6 +101,7 @@ export class WorldModel {
     this.tiles = tiles;
     this.pending_goals = pending_goals;
     this.drop_goals = drop_goals;
+    this.messages = messages;
     this.draw_canvas();
 
     this.robots = robots.map(
@@ -141,6 +144,8 @@ export class WorldModel {
 
     this.ui = {
       wrapper,
+      stageHeight: this.height + this.bs,
+      stageWidth: Math.max(this.width, 500) + this.bs,
       layers: {
         bg: new Konva.Layer({ offsetY: -150 }),
         main: new Konva.Layer({ offsetY: -150 }),
@@ -152,10 +157,12 @@ export class WorldModel {
   draw_canvas() {
     let padding = 25;
     let msgHeight = 150;
+    this.ui.stageHeight = this.height + this.bs + padding + msgHeight;
+    this.ui.stageWidth = Math.max(this.width, 500) + this.bs + padding;
     let stage = new Konva.Stage({
       container: 'container',
-      width: Math.max(this.width, 500) + this.bs + padding,
-      height: this.height + this.bs + padding + msgHeight,
+      width: this.ui.stageWidth,
+      height: this.ui.stageHeight,
     });
 
     this.ui.layers.msg = new Konva.Layer({
@@ -175,6 +182,7 @@ export class WorldModel {
     this.draw_objects();
     this.draw_stats();
     this.draw_msg_containers();
+    this.draw_envelops();
 
     this.draw_drop_goals(this.drop_goals);
 
@@ -365,6 +373,17 @@ export class WorldModel {
     }
   }
 
+  draw_envelops() {
+    for (const key in this.messages) {
+      const [x, y] = key.split(',').map((zz) => parseInt(zz));
+      this.draw_envelop(x, y, this.messages[key]);
+    }
+  }
+
+  draw_envelop(x: number, y: number, message: string) {
+    this.draw_custom('envelope', x, y);
+  }
+
   update_object(x: number, y: number, val: number) {
     let text = this.ui.layers.main.find(`.obj-${x}-${y}-text`)[0];
     if (text) {
@@ -419,11 +438,37 @@ export class WorldModel {
     this.ui.layers.main.draw();
   }
 
+  draw_sprite(sprite_name: string, x: number, y: number, frameRate = 1) {
+    let spritePath = this.tileMap[sprite_name];
+    let [cx, cy] = this.point2cxy(x, y);
+    let sprite = new Image();
+    sprite.src = spritePath;
+    const animations = {
+      motion: [0, 0, 40, 40, 40, 0, 40, 40],
+    };
+    let that = this;
+    sprite.onload = function () {
+      let imageSprite = new Konva.Sprite({
+        x: cx + LEFT_PADDING,
+        y: cy - that.bs / 2,
+        name: `sprite-${x}-${y}-img`,
+        image: sprite,
+        animation: 'motion',
+        animations: animations,
+        frameRate: frameRate,
+      });
+
+      that.ui.layers.main.add(imageSprite);
+      that.ui.layers.main.batchDraw();
+      imageSprite.start();
+    };
+  }
+
   draw_custom(
     obj_name: string,
     x: number,
     y: number,
-    val: number,
+    val: any = null,
     isGoal: boolean = false
   ) {
     let imagePath = this.tileMap[obj_name];
@@ -435,7 +480,7 @@ export class WorldModel {
       y: cy - this.bs / 2 + (this.bs - radius) - MARGIN_NUMBER_CIRCLE,
     });
 
-    if (!isGoal) {
+    if (!isGoal && !!val) {
       let circle = new Konva.Circle({
         radius: radius,
         fill: 'white',
@@ -453,7 +498,6 @@ export class WorldModel {
         offsetX: circle.x() + radius - TEXT_MARGIN,
         offsetY: circle.y() + radius - TEXT_MARGIN,
       });
-
       group.add(circle, num);
     }
 
@@ -627,6 +671,79 @@ export class WorldModel {
     if (border) this.ui.layers.main.add(border);
   }
 
+  read_message(msg: string, waitFor = 3, img = 'envelope-sprite') {
+    let msgBox = new Konva.Text({
+      padding: 20,
+      align: 'center',
+      text: msg,
+      fontSize: 18,
+    });
+
+    let bubble = new Konva.Rect({
+      stroke: '#555',
+      strokeWidth: 5,
+      fill: '#ddd',
+      width: 300,
+      height: msgBox.height(),
+      shadowColor: 'black',
+      shadowBlur: 10,
+      shadowOffsetX: 10,
+      shadowOffsetY: 10,
+      shadowOpacity: 0.2,
+      cornerRadius: 10,
+    });
+
+    let bubbleImg = new Image();
+    bubbleImg.src = this.tileMap['speech_bubble'];
+    bubbleImg.onload = function () {
+      bubble.fillPatternImage(bubbleImg);
+    };
+
+    let group = new Konva.Group({
+      x: 120,
+      y: 510,
+    });
+
+    let tri = new Konva.RegularPolygon({
+      sides: 3,
+      stroke: '#555',
+      strokeWidth: 5,
+      radius: 15,
+      fill: '#555',
+      shadowColor: 'black',
+      shadowBlur: 10,
+      shadowOffsetX: 10,
+      shadowOffsetY: 10,
+      shadowOpacity: 0.2,
+      x: bubble.x() - 5,
+      y: bubble.y() + bubble.height() / 2,
+      rotation: 270,
+    });
+
+    group.add(tri, bubble, msgBox);
+
+    let that = this;
+    this.ui.layers.bg.add(group);
+
+    let tween = new Konva.Tween({
+      node: group,
+      duration: 1,
+      opacity: 0,
+      onFinish: () => {
+        tween.destroy();
+        group.destroy();
+        that.ui.stage?.height(that.ui.stageHeight);
+      },
+    });
+
+    setTimeout(function () {
+      tween.play();
+    }, waitFor * 1000);
+
+    this.ui.stage?.height(this.ui.stageHeight + bubble.height() + 20);
+    this.ui.layers.bg.draw();
+  }
+
   remove_wall(x: number, y: number, dir: string) {
     if (dir !== 'north' && dir !== 'east') return;
     let wall = this.ui.layers.main.find(
