@@ -177,6 +177,14 @@ export class MazeView extends DOMWidgetView {
     return this.world_model.success_msg(msg);
   };
 
+  show_message = (
+    msg: string,
+    waitFor: number = 1,
+    img: string = 'envelope'
+  ) => {
+    return this.world_model.read_message(msg, waitFor, img);
+  };
+
   render() {
     //set methods
     this.method_changed();
