@@ -17,6 +17,7 @@ class WorldParser:
         self.parse_tilemaps()
         self.parse_walls()
         self.parse_tiles()
+        self.parse_messages()
         self.parse_robots()
         self.parse_objects()
         self.parse_goals()
@@ -54,6 +55,13 @@ class WorldParser:
                 parsed_val = self.parse_val(val)
                 [x,y] = map(int, pos.split(","))
                 self.world.add_object(x, y, obj_name, parsed_val)
+    
+
+    def parse_messages(self):
+        messages = self.config.get('messages', {})
+        for pos, msg in messages.items():
+            [x, y] = map(int, pos.split(","))
+            self.world.add_msg(x, y, msg)
 
     def parse_val(self, value):
         if isinstance(value, list):
