@@ -28,7 +28,7 @@ class Robot():
         self.collections = {}
         self.speed  = 0.5
         self.last_picked = []
-        self.count_report = None
+        self.count_report = []
         self.max_capacity = None
         self.move_count = 0
         self.stats = Stats(self)
@@ -56,6 +56,10 @@ class Robot():
     @y.setter
     def y(self, value):
         self.model.y = value
+
+
+    def check(self):
+        self.world.check()
     
     def move(self, step=1):
         """Move  in the direction that the bot is facing. 
@@ -197,7 +201,7 @@ class Robot():
         >>> bot.report("Counted 10 apples")
 
         """
-        self.count_report = msg
+        self.count_report.append(msg)
         print_success("{}".format(msg))
     
     def cell(self):
@@ -276,8 +280,10 @@ class Robot():
         else: 
             return 0
     
-    def read_message(self):
+    def read_message(self, wait_For=1):
         """Check and returns the message in the current cell.
+           Also, it takes an optional parameter in seconds for waiting before it disappears.
+     
 
         Returns
         -------
@@ -289,10 +295,24 @@ class Robot():
         --------
         >>> bot.read_message()
         "Hi There"
+        >>> bot.read_message(3)
+        "Hi There"
         """
         cell = self.cell()
+        if self.message_here():
+            self.world.js_call("show_message", [cell.msg, wait_For])
         return cell.msg 
 
+    def message_here(self):
+        """Check if the bot is on the envelope cotaining message."
+
+        Returns
+        bool
+            Returns True if envelope is present in the current cell.
+            False otherwise.
+        """
+        return self.cell().msg is not None
+
 
     def object_here(self):
         """Check and return the name of the object in the current cell.
